#include <stdio.h>
#include <math.h>
#include <libcw.h>
#include "wiringPi/wiringPi/wiringPi.h"

/* 0-70MHz DDS generator on AD9851 for Raspberry Pi
               Michal Byrecki SP5BYK
		   License: GPL					*/

//   HARDDWARE WIING FOR RASPBERRY PI-2 with 40-pole connector	*/
// Now, the GPIO addresses. This part may be quite confusing, as libwiringPi uses a different
// GPIO descriptors than the pictures published on the Internet (i.e. at RS Components).
// But as far as we're using the libwiringPi, we will stick to the libwiringPi coding.
// The coding way of libwiringPi can be found by executing "pintest" program, that will
// result with a nice, semi-graphic table showing names & states of each GPIO pin.

// GND supplied from 		PIN 39
// Power 3.3V supplied from 	PIN 1
#define DDS_DATA	29	//GPIO 29 (pin 40)
#define DDS_CLK		28	//GPIO 27 (pin 38)
#define	DDS_ENA		25	//GPIO 25 (pin 37), FQ_UPLOAD
#define DDS_RST		24	//GPIO 24 (pin 35)
#define DDS_CLOCK	30000000UL
#define DDS_MUL_TRIGGER	5000000UL
#define DDS_FREQ_MAX	70000000UL

void DDS_outZero (void)
{
digitalWrite (DDS_CLK,0);
digitalWrite (DDS_DATA,0);
digitalWrite (DDS_CLK,1);
}
void DDS_outOne (void)
{
digitalWrite(DDS_CLK,0);
digitalWrite(DDS_DATA,1);
digitalWrite(DDS_CLK,1);
digitalWrite(DDS_DATA,0);
}
void DDS_outByte(unsigned char byte)
{
int i;
for (i=0;i<8;i++)
    {
    if ((byte&1)==1) DDS_outOne();
    else DDS_outZero();
    byte=byte>>1;
    }
}
void DDS_frequency (unsigned long frequency)
{
unsigned long tuning_word;
int i;
    if (frequency>DDS_MUL_TRIGGER) tuning_word=((frequency/6)*pow(2,32))/DDS_CLOCK;
    else tuning_word=((frequency)*pow(2,32))/DDS_CLOCK;
    digitalWrite(DDS_ENA,0);
    for (i=0;i<32;i++)
	{
	if ((tuning_word & 1)==1) DDS_outOne();
	else DDS_outZero();
	tuning_word=tuning_word>>1;

	}
if (frequency>DDS_MUL_TRIGGER) DDS_outByte (0x09);
else DDS_outByte(0x0);
digitalWrite(DDS_ENA,1);
}
void DDS_hwinit (void)
{
pinMode (DDS_DATA,OUTPUT);
pinMode (DDS_CLK,OUTPUT);
pinMode (DDS_ENA,OUTPUT);
pinMode (DDS_RST,OUTPUT);
digitalWrite(DDS_RST,1);
digitalWrite(DDS_RST,0);
}

int main (int argc, char *argv[])
{
unsigned long freqq;
if (argc<2) { printf ("Please use dds <frequency>\n"); return -1; }
sscanf (argv[1],"%lu",&freqq);
if (freqq<0 || freqq>DDS_FREQ_MAX) { printf ("Error: wrong frequency\n"); return -1; }
printf ("Setting freq to %lu\n",freqq) ;
 
if (wiringPiSetup () == -1) { printf ("Error: error initializing Raspi GPIO\n"); return -1; }
DDS_hwinit();
DDS_frequency(freqq);
DDS_frequency(freqq);
return 0 ;
}
